#ifndef SVDOP_H
#define SVDOP_H

#include <algorithm>

namespace Spectra {

    template <typename Scalar, typename MatProd>
    class SVDTallOp
    {
    public:
        SVDTallOp(MatProd* op_) :
            op(op_),
            dim(std::min(op->rows(), op->cols())),
            work(new Scalar[op->rows()])
        {}

        ~SVDTallOp()
        {
            delete[] work;
        }

        int rows() const { return dim; }
        int cols() const { return dim; }

        // y_out = A'A * x_in
        void perform_op(Scalar* x_in, Scalar* y_out)
        {
            op->perform_op(x_in, work);
            op->perform_tprod(work, y_out);
        }

        void perform_tprod(Scalar* x_in, Scalar* y_out)
        {
            perform_op(x_in, y_out);
        }
    private:
        MatProd*  op;
        const int dim;
        Scalar*   work;
    };

    template <typename Scalar, typename MatProd>
    class SVDWideOp
    {
    public:
        SVDWideOp(MatProd* op_) :
            op(op_),
            dim(std::min(op->rows(), op->cols())),
            work(new Scalar[op->cols()])
        {}

        ~SVDWideOp()
        {
            delete[] work;
        }

        int rows() const { return dim; }
        int cols() const { return dim; }

        // y_out = AA' * x_in
        void perform_op(Scalar* x_in, Scalar* y_out)
        {
            op->perform_tprod(x_in, work);
            op->perform_op(work, y_out);
        }

        void perform_tprod(Scalar* x_in, Scalar* y_out)
        {
            perform_op(x_in, y_out);
        }
    private:
        MatProd*  op;
        const int dim;
        Scalar*   work;
    };

}

#endif // SVDOP_H
