#include "tsvd.hpp"
//#include "RedSVD.h"

#include <Eigen/Core>
#include <Eigen/Sparse>
using namespace Eigen;

#include <iostream>
using namespace std;

int main(int argc, char** argv)
{
    int n = 100;
    int k = 7;

    ArrayXi i1 = ArrayXi::LinSpaced(n - 2, 2, n - 1);
    ArrayXi i2 = ArrayXi::LinSpaced(n, 0, n - 1);
    ArrayXi i3 = ArrayXi::LinSpaced(n - 2, 0, n - 3);
    int ni1 = i1.size();
    int ni2 = i2.size();
    int ni3 = i3.size();
    int ni = ni1 + ni2 + ni3;

    ArrayXi i(ni);
    i.head(ni1) = i1;
    i.tail(ni3) = i3;
    i.segment(ni1, ni2) = i2;

    ArrayXi j1 = i3;
    ArrayXi j2 = i2;
    ArrayXi j3 = i1;
    int nj1 = ni3;
    int nj2 = ni2;
    int nj3 = ni1;

    ArrayXi j(ni);
    j.head(nj1) = j1;
    j.tail(nj3) = j3;
    j.segment(nj1, nj2) = j2;

    ArrayXd s(ni);
    s.head(n - 2) = ArrayXd::Ones(n - 2);
    s.tail(n - 2) = ArrayXd::Ones(n - 2);
    s.segment(n - 2, n) = 0.4*n*ArrayXd::Ones(n);

    //Eigen::SparseMatrix<double> A(n, n);
    MatrixXd A = MatrixXd::Zero(n, n);
    for (int index = 0; index < ni; ++index)
    {
        A(i(index), j(index)) = s(index);
        //A.insert(i(index), j(index)) = s(index);
    }

    // Full SVD

    JacobiSVD<MatrixXd> svd(A, ComputeThinU | ComputeThinV);

    cout << "Full SVD" << endl;

    cout << "svd.singularValues():" << endl;
    cout << svd.singularValues() << endl;

    /*
    cout << "U:" << endl;
    cout << svd.matrixU() << endl;

    cout << "V:" << endl;
    cout << svd.matrixV() << endl << endl;*/

#if 0
    // redsvd

    RedSVD::RedSVD<MatrixXd> redsvd(A, k);
    MatrixXd U = redsvd.matrixU();
    MatrixXd S = redsvd.singularValues();
    MatrixXd V = redsvd.matrixV();

    cout << "RedSVD" << endl;
    cout << "S:" << endl;
    cout << S << endl << endl;

    /*
    cout << "U:" << endl;
    cout << U << endl << endl;

    cout << "V:" << endl;
    cout << V << endl << endl;*/

#endif

    int nu = oia::maths::Tsvd<double>::Default;
    int nv = oia::maths::Tsvd<double>::Default;
    int ncv = oia::maths::Tsvd<double>::Default;
    VectorXd d;
    MatrixXd u;
    MatrixXd v;

    int niter;
    int nops;

    oia::maths::Tsvd<double> tsvd;
    tsvd.tsvd_gen(A, k, nu, nv, ncv, d, u, v, niter, nops);

    //for (int i = 0; i < A.outerSize(); ++i)
    //{
    //    for (SparseMatrix<double>::InnerIterator it(A, i); it; ++it)
    //    {
    //        //  (1, 1) ->  40
    //        cout << "(" << it.row() << ", " << it.col() << ") -> " << it.value() << endl;
    //    }
    //}

    cout << "Spectra" << endl;
    cout << "d:" << endl;
    cout << d << endl << endl;

    /*
    cout << "u:" << endl;
    cout << u << endl << endl;

    cout << "v:" << endl;
    cout << v << endl << endl;*/
	
	return 0;
}